local addonName, TF = ...

local TradeFill = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0")
local Loc = LibStub("AceLocale-3.0"):GetLocale(addonName)

local Minimap = LibStub("LibDBIcon-1.0", true)

local OptionFrame = CreateFrame("Frame", "MyIconButtonFrame", TradeFrame)
local TradeButtonClear = CreateFrame("Button", "MyTradeButton", OptionFrame, "UIPanelButtonTemplate")

function TradeFill:OnInitialize()
    TF.signature = "|c8788EEff" .. addonName .. ":|r "

    TF.interfaceItem = {}
    TF.stackCount = {}
    TF.messages = {}

    local defaultsItems = {
        profile = {
            item = {
                ["*"] = {
                    id = 0,
                    name = "",
                    link = "",
                    quality = 0,
                    level = 0,
                    stack = 0,
                    texture = 0,
                },
            },
            stacks = {
                ["*"] = {
                    ["1"] = 0,
                    ["2"] = 0,
                    ["3"] = 0,
                    ["4"] = 0,
                    ["5"] = 0,
                    ["6"] = 0,
                },
            },
            setting = {
                solo = false,
                party = false,
                raid = false,
                auto = true,
                show = true,
                full = true,
                button = false,
                message = false,
                require = false,
                level = false,
                master = false,
                guild = false,
                guilds = "",
                players = "",
            },
            minimap = {
                hide = false,
            },
        },
    }

    local tradeFillHistory = {
        profile = {
            ["*"] = {
                player = "",
                tradePlayer = {
                    money = "",
                    ["*"] = {
                        ["*"] = {
                            id = 0,
                            name = "",
                            texture = 0,
                            quantity = 0,
                            link = ""
                        },
                    },
                },
                target = "",
                tradeTarget = {
                    money = "",
                    ["*"] = {
                        ["*"] = {
                            id = 0,
                            name = "",
                            texture = 0,
                            quantity = 0,
                            link = ""
                        },
                    },
                },
            },
        },
    }

    self.db = LibStub("AceDB-3.0"):New("TradeFillDB", defaultsItems)
    self.history = LibStub("AceDB-3.0"):New("TradeFillHistoryDB", tradeFillHistory)

    TF.options = {
        name = addonName,
        handler = TradeFill,
        type = "group",
        childGroups = "tab",
        args = {
            menu = {
                type = "group",
                name = Loc["TAB_ITEMS"],
                desc = Loc["TAB_ITEMS_DESC"],
                order = 1,
                args = {
                    toggleBoxA = {
                        type = 'toggle',
                        name = Loc["TOGGLE_SHOW"],
                        desc = Loc["TOGGLE_SHOW_DESC"],
                        get = function(info) return TradeFill.db.profile.setting.show end,
                        set = function(info, value) TradeFill.db.profile.setting.show = value end,
                        order = 0,
                    },
                    toggleBoxB = {
                        type = 'toggle',
                        name = Loc["TOGGLE_FULL_STACKS"],
                        desc = Loc["TOGGLE_FULL_STACKS_DESC"],
                        get = function(info) return TradeFill.db.profile.setting.full end,
                        set = function(info, value) TradeFill.db.profile.setting.full = value end,
                        order = 2,
                    },
                    toggleBoxC = {
                        type = 'toggle',
                        name = Loc["TOGGLE_AUTOFILL"],
                        desc = Loc["TOGGLE_AUTOFILL_DESC"],
                        get = function(info) return TradeFill.db.profile.setting.auto end,
                        set = function(info, value) return TradeFill:SetAutoFill(value) end,
                        order = 3,
                    },
                    toggleBoxD = {
                        type = 'toggle',
                        name = Loc["TOGGLE_MINIMAP"],
                        desc = Loc["TOGGLE_MINIMAP_DESC"],
                        get = function(info) return TradeFill.db.profile.minimap.hide end,
                        set = function() return TradeFill:SetMinimapButton() end,
                        order = 16,
                    },
                    toggleBoxE = {
                        type = 'toggle',
                        name = Loc["TOGGLE_BUTTON"],
                        desc = Loc["TOGGLE_BUTTON_DESC"],
                        get = function(info) return TradeFill.db.profile.setting.button end,
                        set = function(info, value) TradeFill.db.profile.setting.button = value end,
                        order = 17,
                    },
                    toggleBoxF = {
                        type = 'toggle',
                        name = Loc["TOGGLE_MESSAGES"],
                        desc = Loc["TOGGLE_MESSAGES_DESC"],
                        get = function(info) return TradeFill.db.profile.setting.message end,
                        set = function(info, value) TradeFill.db.profile.setting.message = value end,
                        order = 18,
                    },
                },
            },
            stacks = {
                type = "group",
                name = Loc["TAB_STACKS"],
                desc = Loc["TAB_STACKS_DESC"],
                order = 2,
                args = {
                },
            },
            group = {
                type = "group",
                name = Loc["TAB_CONDITION"],
                desc = Loc["TAB_CONDITION_DESC"],
                order = 3,
                args = {
                    group2 = {
                        type = "group",
                        name = Loc["TREE_CONDITION"],
                        desc = string.format(Loc["TREE_CONDITION_DESC"], TradeFill:GetMaxPlayerLevel()),
                        order = 1,
                        args = {
                            toggleBoxA = {
                                type = 'toggle',
                                name = Loc["TOGGLE_STRANGER"],
                                desc = Loc["TOGGLE_STRANGER_DESC"],
                                get = function() return TradeFill.db.profile.setting.solo end,
                                set = function(info, value) TradeFill.db.profile.setting.solo = value end,
                                order = 1,
                            },
                            toggleBoxB = {
                                type = 'toggle',
                                name = Loc["TOGGLE_PARTY"],
                                desc = Loc["TOGGLE_PARTY_DESC"],
                                get = function() return TradeFill.db.profile.setting.party end,
                                set = function(info, value) TradeFill.db.profile.setting.party = value end,
                                order = 2,
                            },
                            toggleBoxC = {
                                type = 'toggle',
                                name = Loc["TOGGLE_RAID"],
                                desc = Loc["TOGGLE_RAID_DESC"],
                                get = function() return TradeFill.db.profile.setting.raid end,
                                set = function(info, value) TradeFill.db.profile.setting.raid = value end,
                                order = 3,
                            },
                            separatorA = {
                                type = "description",
                                name = " ",
                                order = 4,
                            },
                            toggleBoxD = {
                                type = 'toggle',
                                name = Loc["TOGGLE_GUILD"],
                                desc = Loc["TOGGLE_GUILD_DESC"],
                                get = function() return TradeFill.db.profile.setting.guild end,
                                set = function(info, value) TradeFill.db.profile.setting.guild = value end,
                                order = 5,
                            },
                            toggleBoxE = {
                                type = 'toggle',
                                name = Loc["TOGGLE_REQUIRED"],
                                desc = Loc["TOGGLE_REQUIRED_DESC"],
                                get = function() return TradeFill.db.profile.setting.require end,
                                set = function(info, value) TradeFill.db.profile.setting.require = value end,
                                order = 6,
                            },
                            toggleBoxF = {
                                type = 'toggle',
                                name = function() return TradeFill:GetMaxPlayerLevel() end,
                                desc = Loc["TOGGLE_LEVEL_DESC"],
                                get = function() return TradeFill.db.profile.setting.level end,
                                set = function(info, value) TradeFill.db.profile.setting.level = value end,
                                order = 7,
                            },
                            separatorB = {
                                type = "description",
                                name = " ",
                                order = 8,
                            },
                            toggleBoxG = {
                                type = 'toggle',
                                name = Loc["TOGGLE_MASTER"],
                                desc = Loc["TOGGLE_MASTER_DESC"],
                                get = function() return TradeFill.db.profile.setting.master end,
                                set = function(info, value) TradeFill.db.profile.setting.master = value end,
                                order = 9,
                            },
                        },
                    },
                    player = {
                        type = "group",
                        name = Loc["TREE_PLAYERS"],
                        desc = Loc["TREE_PLAYERS_DESC"],
                        order = 2,
                        args = {
                            inputA = {
                                type = 'input',
                                name = Loc["INPUT_TEXTAREA_PLAYERS"],
                                desc = Loc["INPUT_TEXTAREA_PLAYERS_DESC"],
                                get = function() return TradeFill.db.profile.setting.players end,
                                set = function(info, value) TradeFill.db.profile.setting.players = value:gsub("\n*$", "") end,
                                pattern = "^[A-Za-z\n]*$",
                                usage = Loc["INPUT_TEXTAREA_PLAYERS_USAGE"],
                                multiline = 15,
                                order = 1,
                            },
                        },
                    },
                    guild = {
                        type = "group",
                        name = Loc["TREE_GUILDS"],
                        desc = Loc["TREE_GUILDS_DESC"],
                        order = 3,
                        args = {
                            inputA = {
                                type = 'input',
                                name = Loc["INPUT_TEXTAREA_GUILDS"],
                                desc = Loc["INPUT_TEXTAREA_GUILDS_DESC"],
                                get = function() return TradeFill.db.profile.setting.guilds end,
                                set = function(info, value) TradeFill.db.profile.setting.guilds = value:gsub("\n*$", "") end,
                                pattern = "^[A-Za-z\n ]*$",
                                usage = Loc["INPUT_TEXTAREA_GUILDS_USAGE"],
                                multiline = 15,
                                order = 1,
                            },
                        },
                    },
                },
            },
            logbook = {
                type = "group",
                name = Loc["TAB_HISTORY"],
                desc = Loc["TAB_HISTORY_DESC"],
                order = 4,
                args = {
                },
            },
        },
    }

    for i = 1, MAX_TRADABLE_ITEMS do
        TF.stackCount[i] = { id = 0, trade = 0 }
    end

    for index, class in pairs(classes) do
        TF.options.args.stacks.args[class] = {
            type = "group",
            name = TradeFill:SetClass(class),
            order = index,
            args = {
            },
        }
    end

    LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, TF.options)
    local AceConfigDialog = LibStub("AceConfigDialog-3.0")
    AceConfigDialog:AddToBlizOptions(addonName, addonName)

    TradeFill:InitializeInterfaceOption()
    TradeFill:InitializeInterfaceOptionStack()
    TradeFill:InitializeHistory()

    Minimap:InitializeMinimapButton(self)

    OptionFrame:SetWidth(200)
    OptionFrame:SetHeight(600)
    OptionFrame:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0)

    TradeFill:InitializeTradeButton()
end

function TradeFill:OnEnable()
    self:RegisterChatCommand("tradefill", "SlashCommand")
    self:RegisterChatCommand("tradefillon", "On")
    self:RegisterChatCommand("tradefilloff", "Off")

    self:RegisterEvent("TRADE_SHOW")
    self:RegisterEvent("BAG_UPDATE_DELAYED")

    DEFAULT_CHAT_FRAME:AddMessage(string.format(TF.signature .. Loc["VERSION"], C_AddOns.GetAddOnMetadata(addonName, "Version")))
end

function TradeFill:InitializeInterfaceOption()
    local optionIndex = 4

    for i = 1, MAX_TRADABLE_ITEMS do
        TF.options.args.menu.args[tostring(optionIndex)] = {
            type = "description",
            name = " ",
            order = optionIndex,
        }

        TF.options.args.menu.args[tostring(optionIndex + 1)] = {
            type = "select",
            width = 2,
            name = function() return TradeFill:GetName(tostring(i)) end,
            desc = Loc["OPTIONS_ITEM_DESC"],
            values = function() return TradeFill:SetOptionItem(tostring(i)) end,
            get = function() return TradeFill:GetInputItem(tostring(i)) end,
            set = function(info, value) TradeFill:SetSelectItem(tostring(i), value) end,
            order = optionIndex + 1,
        }
        optionIndex = optionIndex + 2
    end
    TF.options.args.menu.args[tostring(optionIndex)] = {
        type = "description",
        name = " ",
        order = optionIndex,
    }
end

function TradeFill:InitializeInterfaceOptionStack()
    local classIndex = 1

    for _, class in pairs(classes) do
        for i = 1, MAX_TRADABLE_ITEMS do
            TF.options.args.stacks.args[class].args[tostring(classIndex)] = {
                type = "description",
                fontSize = "medium",
                name = " ",
                order = classIndex,
            }

            TF.options.args.stacks.args[class].args[tostring(classIndex + 1)] = {
                type = "select",
                width = 1.5,
                name = function() return TradeFill:GetName(tostring(i)) end,
                desc = Loc["TREE_STACKS_SELECT_DESC"],
                values = function() return TradeFill:SetOptionStack() end,
                get = function() return tostring(TradeFill.db.profile.stacks[class][tostring(i)]) end,
                set = function(info, value) TradeFill.db.profile.stacks[class][tostring(i)] = tonumber(value) end,
                order = classIndex + 1,
            }
            classIndex = classIndex + 2
        end
        classIndex = 1
    end
end

function TradeFill:GetSelectedItems()
    if next(TF.interfaceItem) == nil then
        for bag = 0, 4 do
            for slot = 1, C_Container.GetContainerNumSlots(bag) do
                local bagItem = C_Container.GetContainerItemInfo(bag, slot)

                if bagItem ~= nil then
                    table.insert(TF.interfaceItem, bagItem)
                end
            end
        end
    end
end

function TradeFill:SetOptionItem(option)

    TradeFill:GetSelectedItems()

    local selectOption = {}

    selectOption["0"] = Loc["OPTION_NONE"]

    for i, slot in pairs(TF.interfaceItem) do

        local _, _, _, _, _, _, _, itemStackCount = GetItemInfo(slot.itemID)

        if TradeFill.db.profile.setting.show == true then
            if slot.isBound == false and slot.stackCount == itemStackCount then
                selectOption[slot.itemName] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        else
            if slot.isBound == false then
                selectOption[slot.itemName] = "|T" .. GetItemIcon(slot.itemID) .. ":0|t " .. slot.hyperlink:gsub("[%[%]]", "")
            end
        end

        if TradeFill.db.profile.item[option].id > 0 then
            local item = TradeFill.db.profile.item[option]

            selectOption[item.name] = "|T" .. GetItemIcon(item.id) .. ":0|t " .. item.link:gsub("[%[%]]", "")
        end
    end

    return selectOption
end

function TradeFill:GetName(option)
    local level = TradeFill.db.profile.setting.require
    local item = self.db.profile.item[option]

    if item.id > 0 then
        if item.link == "" or
           item.name == "" or
           item.texture == 0 or
           item.stack == 0 then

            local itemID = TradeFill.db.profile.item[tostring(option)].id
            TradeFill:SetSelectItem(tostring(option), itemID)
            return
        end
    end

    if item.id > 0 then
        if level == true then
            return "|T" .. GetItemIcon(item.id) .. ":24[:24[:2:3|t  " .. item.link:gsub("[%[%]]", "") .. "|cFFFFD100" .. Loc["ITEM_NAME"] .. " " .. item.level
        else
            return "|T" .. GetItemIcon(item.id) .. ":24[:24[:2:3|t  " .. item.link:gsub("[%[%]]", "")
        end
    else
        return "|T" .. GetItemIcon(1217) .. ":24[:24[:2:3|t  " .. Loc["ITEM_UNKNOWN"]
    end
end

function TradeFill:SetSelectItem(option, selctedItemName)
    if selctedItemName == "0" then
        self.db.profile.item[option] = nil
        return
    else
        local _, itemLink, itemQuality, _, itemMinLevel, _, _, itemStackCount, _, itemTexture = GetItemInfo(selctedItemName)

        TradeFill:GetSelectedItems()

        local function GetID(name)
            for key, _ in pairs(TF.interfaceItem) do
                if TF.interfaceItem[key].itemName == name then
                    return TF.interfaceItem[key].itemID
                end
            end
        end

        local set = self.db.profile.item[option]
        set.id = GetID(selctedItemName)
        set.name = selctedItemName
        set.link = itemLink:gsub("%[.+%]", "[" .. selctedItemName .. "]")
        set.quality = itemQuality
        set.level = itemMinLevel
        set.stack = itemStackCount
        set.texture = itemTexture
    end
end

function TradeFill:GetInputItem(option)
    local item = TradeFill.db.profile.item[option]

    TradeFill:SetClassOption(item.id, option)
    return item.name
end

function TradeFill:InitializeTradeButton()
    TF.TradeButtons = {}

    TradeButtonClear:SetText("Clear")
    TradeButtonClear:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 73, 5)
    TradeButtonClear:SetSize(75, 22)
    TradeButtonClear:SetScript("OnClick", function() TradeFill:ClearTade() end)

    local buttonOrder = { 28, 62, 96, 130, 164, 198 }

    for index, name in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
        local button = CreateFrame("Button", name, OptionFrame)
        button:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 34, buttonOrder[index])
        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetSize(32, 32)

        button.label = button:CreateFontString(nil, "OVERLAY", "NumberFontNormal")
        button.label:SetPoint("BOTTOMRIGHT", OptionFrame, "BOTTOMRIGHT", 34, buttonOrder[index])

        TF.TradeButtons[name] = button
    end
end

function TradeFill:ButtonInitialize()
    for index, name in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
    local item = TradeFill.db.profile.item[tostring(index)]

        if item.id > 0 then
            TradeFill:SetButton()
            TF.TradeButtons[name].label:SetText(GetItemCount(item.id, false))
            TF.TradeButtons[name]:SetNormalTexture(GetItemIcon(item.id))
            TF.TradeButtons[name]:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
            TF.TradeButtons[name]:SetScript("OnClick", function(self, button)
                if button == "LeftButton" then
                    TradeFill:AutoFill(item, TradeFrame_GetAvailableSlot())
                elseif button == "RightButton" then
                    TradeFill:RemoveItem(item)
                end
            end)

            TF.TradeButtons[name]:SetScript("OnEnter", function(self)
                GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
                GameTooltip:AddLine(item.link:gsub("[%[%]]", ""))
                GameTooltip:AddLine(string.format(Loc["BUTTON_TOOLTIP"]), 1, 1, 1)
                GameTooltip:Show()
            end)
            TF.TradeButtons[name]:SetScript("OnLeave", function()
                GameTooltip:Hide()
            end)
        else
            TF.TradeButtons[name]:Hide()
        end
    end
end

function TradeFill:ButtonUpdate()
    TF.messages = {}

    for i = 1, MAX_TRADABLE_ITEMS do
        local item = TradeFill.db.profile.item[tostring(i)]

        if item.id > 0 then
            for key, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
                if i == key then
                    TF.TradeButtons[button].label:SetText(tostring(GetItemCount(item.id, false) - TF.stackCount[i].trade))
                end
            end
        end
    end
end

function TradeFill:SetAutoFill()
    if TradeFill.db.profile.setting.auto then
        self.db.profile.setting.auto = false
        Minimap:SetButtonOff()
    else
        self.db.profile.setting.auto = true
        Minimap:SetButtonOn()
    end
end

function TradeFill:SetMinimapButton()
    if TradeFill.db.profile.minimap.hide then
        self.db.profile.minimap.hide = false
        Minimap:Show("minimap")
    else
        self.db.profile.minimap.hide = true
        Minimap:Hide("minimap")
    end
end

function TradeFill:SetClassOption(value, option)
    local key = tostring(tonumber(option * 2))

    if value == 0 then
        for _, class in pairs(classes) do
            TF.options.args.stacks.args[class].args[key].disabled = true
        end
    else
        for _, class in pairs(classes) do
            TF.options.args.stacks.args[class].args[key].disabled = false
        end
    end
end

function TradeFill:SetButton()
    local status = TradeFill.db.profile.setting.button

    if status then
        for _, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
            TF.TradeButtons[button]:Hide()
            TradeButtonClear:Hide()
        end
    else
        for _, button in pairs( { "TradeFillA", "TradeFillB", "TradeFillC", "TradeFillD", "TradeFillE", "TradeFillF" } ) do
            TF.TradeButtons[button]:Show()
            TradeButtonClear:Show()
        end
    end
end

function TradeFill:SetOptionStack()
    local selectOption = {}

    for i = 0, MAX_TRADABLE_ITEMS do
        selectOption[tostring(i)] = i
    end

    return selectOption
end

function TradeFill:GetMaxPlayerLevel()
    return string.format(Loc["TOGGLE_LEVEL"], GetMaxPlayerLevel())
end

function TradeFill:SetClass(classFile)
    local className = LOCALIZED_CLASS_NAMES_MALE[classFile]
    local _, _, _, argbHex = GetClassColor(classFile)

    if classFile == "SHAMAN" then
        argbHex = "FF0070DE"
    end

    return "|c" .. argbHex .. className .. "|r"
end

function TradeFill:GetItemColor(quality, item)
    local _, _, _, hex = GetItemQualityColor(quality)

    return "|c" .. hex .. item .. "|r"
end

function TradeFill:SlashCommand()
    InterfaceOptionsFrame_OpenToCategory(TradeFill)
end

function TradeFill:ClearTade()
    for tradeSlot = 1, MAX_TRADABLE_ITEMS do
        ClearCursor()
        ClickTradeButton(tradeSlot)
        ClearCursor()
    end

    TradeFill:ButtonUpdate()
end

function TradeFill:On()
    TradeFill.db.profile.setting.auto = true

    Minimap:SetButtonOn()
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["ADDON_ON"])
end

function TradeFill:Off()
    TradeFill.db.profile.setting.auto = false

    Minimap:SetButtonOff()
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["ADDON_OFF"])
end